@@ -16,15 +16,17 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
-import unittest
+from textwrap import dedent
 
 import mock
 import io
 
-from gitlab import config
+from gitlab import config, USER_AGENT
 import pytest
 
 
+custom_user_agent = "my-package/1.0.0"
+
 valid_config = u"""[global]
 default = one
 ssl_verify = true
@@ -51,6 +53,17 @@ url = https://four.url
 oauth_token = STUV
 """
 
+custom_user_agent_config = """[global]
+default = one
+user_agent = {}
+
+[one]
+url = http://one.url
+private_token = ABCDEF
+""".format(
+    custom_user_agent
+)
+
 no_default_config = u"""[global]
 [there]
 url = http://there.url
@@ -140,9 +153,9 @@ def test_valid_data(m_open, path_exists):
     assert "one" == cp.gitlab_id
     assert "http://one.url" == cp.url
     assert "ABCDEF" == cp.private_token
-    assert None == cp.oauth_token
+    assert cp.oauth_token is None
     assert 2 == cp.timeout
-    assert True == cp.ssl_verify
+    assert cp.ssl_verify is True
     assert cp.per_page is None
 
     fd = io.StringIO(valid_config)
@@ -152,9 +165,9 @@ def test_valid_data(m_open, path_exists):
     assert "two" == cp.gitlab_id
     assert "https://two.url" == cp.url
     assert "GHIJKL" == cp.private_token
-    assert None == cp.oauth_token
+    assert cp.oauth_token is None
     assert 10 == cp.timeout
-    assert False == cp.ssl_verify
+    assert cp.ssl_verify is False
 
     fd = io.StringIO(valid_config)
     fd.close = mock.Mock(return_value=None)
@@ -163,7 +176,7 @@ def test_valid_data(m_open, path_exists):
     assert "three" == cp.gitlab_id
     assert "https://three.url" == cp.url
     assert "MNOPQR" == cp.private_token
-    assert None == cp.oauth_token
+    assert cp.oauth_token is None
     assert 2 == cp.timeout
     assert "/path/to/CA/bundle.crt" == cp.ssl_verify
     assert 50 == cp.per_page
@@ -174,7 +187,62 @@ def test_valid_data(m_open, path_exists):
     cp = config.GitlabConfigParser(gitlab_id="four")
     assert "four" == cp.gitlab_id
     assert "https://four.url" == cp.url
-    assert None == cp.private_token
+    assert cp.private_token is None
     assert "STUV" == cp.oauth_token
     assert 2 == cp.timeout
-    assert True == cp.ssl_verify
+    assert cp.ssl_verify is True
+
+
+@mock.patch("os.path.exists")
+@mock.patch("builtins.open")
+def test_data_from_helper(m_open, path_exists, tmp_path):
+    helper = tmp_path / "helper.sh"
+    helper.write_text(
+        dedent(
+            """\
+            #!/bin/sh
+            echo "secret"
+            """
+        )
+    )
+    helper.chmod(0o755)
+
+    fd = io.StringIO(
+        dedent(
+            """\
+            [global]
+            default = helper
+
+            [helper]
+            url = https://helper.url
+            oauth_token = helper: %s
+            """
+        )
+        % helper
+    )
+
+    fd.close = mock.Mock(return_value=None)
+    m_open.return_value = fd
+    cp = config.GitlabConfigParser(gitlab_id="helper")
+    assert "helper" == cp.gitlab_id
+    assert "https://helper.url" == cp.url
+    assert cp.private_token is None
+    assert "secret" == cp.oauth_token
+
+
+@mock.patch("os.path.exists")
+@mock.patch("builtins.open")
+@pytest.mark.parametrize(
+    "config_string,expected_agent",
+    [
+        (valid_config, USER_AGENT),
+        (custom_user_agent_config, custom_user_agent),
+    ],
+)
+def test_config_user_agent(m_open, path_exists, config_string, expected_agent):
+    fd = io.StringIO(config_string)
+    fd.close = mock.Mock(return_value=None)
+    m_open.return_value = fd
+
+    cp = config.GitlabConfigParser()
+    assert cp.user_agent == expected_agent
